Attribute VB_Name = "modIMFMediaEngineNotify"
' Autor: F. Schler (frank@activevb.de)
' Datum: 11/2020
'
' alle mit einem Underscore gekennzeichneten Const, Types,
' Variablen und APIs werden nur intern verwendet!!!

Option Explicit

' ----==== Const ====----
Private Const S_OK_ As Long = &H0&
Private Const E_NOINTERFACE_ As Long = &H80004002
Private Const IID_IMFMediaEngineNotify_ As String = "{FEE7C112-E776-42B5-9BBF-0048524E2BD5}"

' ----==== Types ====----
Private Type GUID_
    data1 As Long
    data2 As Integer
    data3 As Integer
    data4(0 To 7) As Byte
End Type

Private Type tIMFMediaEngineNotify_
    pVTable As Long
End Type

Private Type tIMFMediaEngineNotifyVTable_
    VTable(0 To 3) As Long
End Type

' ----==== Ole32 API-Deklarationen ====----
Private Declare Sub CoTaskMemFree_ Lib "ole32.dll" _
                    Alias "CoTaskMemFree" ( _
                    ByVal hMem As Long)
                    
Private Declare Function StringFromGUID_ Lib "ole32.dll" _
                         Alias "StringFromIID" ( _
                         ByRef pCLSID As GUID_, _
                         ByRef lpszProgID As Long) As Long
                         
' ----==== Kernel32.dll Deklarationen ====----
Private Declare Sub CopyMemory_ Lib "kernel32.dll" _
                    Alias "RtlMoveMemory" ( _
                    ByRef pTo As Any, _
                    ByRef uFrom As Any, _
                    ByVal lSize As Long)
                    
Private Declare Function lstrlenW_ Lib "kernel32.dll" _
                         Alias "lstrlenW" ( _
                         ByVal lpString As Long) As Long
                         
' ----==== Variablen ====----
Public m_EventNotify As Collection

Private m_Event_(2) As Long
Private m_RefCount_ As Long
Private m_pIMFMediaEngineNotify_ As Long
Private m_IMFMediaEngineNotify_ As tIMFMediaEngineNotify_
Private m_IMFMediaEngineNotifyVTable_ As tIMFMediaEngineNotifyVTable_

Public Function pIMFMediaEngineNotify_() As Long

    ' ----==== Interface IMFMediaEngineNotify  ====----
    With m_IMFMediaEngineNotifyVTable_
        
        ' VTable aufbauen
        .VTable(0) = ProcPtr_(AddressOf QueryInterface_)
        .VTable(1) = ProcPtr_(AddressOf AddRef_)
        .VTable(2) = ProcPtr_(AddressOf Release_)
        .VTable(3) = ProcPtr_(AddressOf EventNotify_)

    End With

    With m_IMFMediaEngineNotify_

        ' Pointer auf die VTable
        .pVTable = VarPtr(m_IMFMediaEngineNotifyVTable_)

    End With
    
    m_pIMFMediaEngineNotify_ = VarPtr(m_IMFMediaEngineNotify_)
    
    ' Pointer auf das Interface
    pIMFMediaEngineNotify_ = m_pIMFMediaEngineNotify_

End Function

Private Function ProcPtr_(ByVal ptr As Long) As Long

    ProcPtr_ = ptr

End Function

' ----==== Interface IUnknown Func ====----
Private Function QueryInterface_(ByVal this As Long, ByRef riid As GUID_, ByRef _
    pvObj As Long) As Long

    Dim lRet As Long
    
    Select Case UCase$(Guid2Str_(riid))

    ' wenn nach dem Interface IMFMediaEngineNotify gefragt wird
    Case UCase$(IID_IMFMediaEngineNotify_)

        ' dann muss ein AddRef aufgerufen werden.
        Call AddRef_(this)
        
        ' Pointer auf das Interface
        pvObj = m_pIMFMediaEngineNotify_
        
        ' OK zurck geben, Interface ist vorhanden
        lRet = S_OK_

    Case Else
        
        ' alles andere ignorieren
        
        pvObj = 0&
        
        lRet = E_NOINTERFACE_
    
    End Select

    QueryInterface_ = lRet

End Function

Private Function AddRef_(ByVal this As Long) As Long

    m_RefCount_ = m_RefCount_ + 1

    AddRef_ = m_RefCount_

End Function

Private Function Release_(ByVal this As Long) As Long

    m_RefCount_ = m_RefCount_ - 1
   
    Release_ = m_RefCount_

End Function

' ----==== Interface IMFMediaEngineNotify Func ====----
Private Function EventNotify_(ByVal this As Long, _
    ByVal lEvent As Long, ByVal Param1 As Long, _
    ByVal Param2 As Long) As Long
    
    m_Event_(0) = lEvent
    m_Event_(1) = Param1
    m_Event_(2) = Param2
    
    Call m_EventNotify.Add(m_Event_)
    
End Function

' ----==== GUID -> String ====----
Private Function Guid2Str_(ByRef tGUID As GUID_) As String

    Dim pStr As Long
    Dim strRet As String

    If StringFromGUID_(tGUID, pStr) = S_OK_ Then

        strRet = Ptr2Str_(pStr)
    
    End If

    Guid2Str_ = strRet

End Function

' ----==== Pointer -> String ====----
Private Function Ptr2Str_(ByVal pStr As Long) As String

    Dim lngLen As Long
    Dim strRet As String
    Dim bytBuffer() As Byte

    If pStr = 0 Then Exit Function

    lngLen = lstrlenW_(pStr) * 2& - 1&

    If lngLen > 0& Then

        ReDim bytBuffer(lngLen)

        Call CopyMemory_(bytBuffer(0), ByVal pStr, lngLen)
        
        Call CoTaskMemFree_(pStr)

        strRet = bytBuffer

    End If
    
    Ptr2Str_ = strRet

End Function
